import mimetypes
import os
import uuid

from fastapi import status
from fastapi.exceptions import HTTPException
from fastapi.responses import FileResponse, JSONResponse
from square_commons import get_api_output_in_standard_format
from square_database_helper import FiltersV0
from square_database_helper.pydantic_models import FilterConditionsV0
from square_database_structure.square import global_string_database_name
from square_database_structure.square.file_storage import global_string_schema_name
from square_database_structure.square.file_storage.tables import File

from square_file_store.configuration import (
    global_absolute_path_local_storage,
    global_object_square_logger,
)
from square_file_store.messages import messages
from square_file_store.utils.helper import (
    create_entry_in_file_store,
    local_object_square_database_helper,
    get_file_row,
)


@global_object_square_logger.auto_logger()
async def util_upload_file_v0(file, app_id, system_relative_path):
    system_file_absolute_path = None
    file_storage_token = None
    try:
        """
        validation
        """
        # pass
        """
        main
        """
        file_bytes = await file.read()
        filename = file.filename
        content_type = file.content_type

        file_storage_token = str(uuid.uuid4())
        system_file_name = str(uuid.uuid4())
        file_extension = filename.rsplit(".", 1)[-1]
        system_file_name_with_extension = system_file_name + "." + file_extension
        response = create_entry_in_file_store(
            file_name_with_extension=filename,
            content_type=content_type,
            file_storage_token=file_storage_token,
            app_id=app_id,
            system_relative_path=system_relative_path,
            system_file_name_with_extension=system_file_name_with_extension,
        )

        # create folder
        system_absolute_path = os.path.join(
            global_absolute_path_local_storage,
            os.sep.join(system_relative_path.split("/")),
        )

        # create path if it doesn't exist.
        if not os.path.exists(system_absolute_path):
            os.makedirs(system_absolute_path)

        system_file_absolute_path = os.path.join(
            system_absolute_path, system_file_name_with_extension
        )
        with open(system_file_absolute_path, "wb") as file:
            file.write(file_bytes)

        # Check if the file exists
        if not os.path.exists(system_file_absolute_path):
            output_content = get_api_output_in_standard_format(
                message=messages["GENERIC_500"],
                log=f"file creation failed at {system_file_absolute_path}.",
            )
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=output_content,
            )
        """
        return value
        """
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_CREATION_SUCCESSFUL"],
            data={"main": response[0][File.file_storage_token.name]},
        )
        return JSONResponse(content=output_content, status_code=status.HTTP_201_CREATED)
    except HTTPException as http_exception:
        """
        rollback logic
        """
        global_object_square_logger.logger.error(http_exception, exc_info=True)
        if os.path.exists(system_file_absolute_path):
            os.remove(system_file_absolute_path)
        if file_storage_token:
            local_object_square_database_helper.delete_rows_v0(
                database_name=global_string_database_name,
                schema_name=global_string_schema_name,
                table_name=File.__tablename__,
                filters=FiltersV0(
                    root={
                        File.file_storage_token.name: FilterConditionsV0(
                            eq=file_storage_token
                        ),
                    }
                ),
            )
        return JSONResponse(
            status_code=http_exception.status_code, content=http_exception.detail
        )
    except Exception as e:
        """
        rollback logic
        """
        if system_file_absolute_path and os.path.exists(system_file_absolute_path):
            os.remove(system_file_absolute_path)
        if file_storage_token:
            local_object_square_database_helper.delete_rows_v0(
                database_name=global_string_database_name,
                schema_name=global_string_schema_name,
                table_name=File.__tablename__,
                filters=FiltersV0(
                    root={
                        File.file_storage_token.name: FilterConditionsV0(
                            eq=file_storage_token
                        ),
                    }
                ),
            )
        global_object_square_logger.logger.error(e, exc_info=True)
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_500"],
            log=str(e),
        )
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, content=output_content
        )


@global_object_square_logger.auto_logger()
def util_download_file_v0(file_storage_token):
    file_storage_token = str(file_storage_token)
    try:
        """
        validation
        """
        # pass
        """
        main
        """
        local_dict_file_row = get_file_row(file_storage_token)
        local_string_system_absolute_file_path = str(
            os.path.join(
                global_absolute_path_local_storage,
                os.sep.join(
                    local_dict_file_row[File.file_system_relative_path.name].split("/")
                ),
                local_dict_file_row[File.file_system_file_name_with_extension.name],
            )
        )
        if not os.path.exists(local_string_system_absolute_file_path):
            output_content = get_api_output_in_standard_format(
                message=messages["GENERIC_500"],
                log=f"file: {file_storage_token} not found in path {local_string_system_absolute_file_path}.",
            )
            raise HTTPException(
                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                detail=output_content,
            )
        """
        return value
        """
        content_type, _ = mimetypes.guess_type(local_string_system_absolute_file_path)

        return FileResponse(
            local_string_system_absolute_file_path,
            media_type=content_type,
            filename=local_dict_file_row[File.file_name_with_extension.name],
        )
    except HTTPException as http_exception:
        """
        rollback logic
        """
        # pass
        global_object_square_logger.logger.error(http_exception, exc_info=True)
        return JSONResponse(
            status_code=http_exception.status_code, content=http_exception.detail
        )
    except Exception as e:
        """
        rollback logic
        """
        # pass
        global_object_square_logger.logger.error(e, exc_info=True)
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_500"],
            log=str(e),
        )
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, content=output_content
        )


@global_object_square_logger.auto_logger()
def util_delete_files_v0(file_storage_tokens):
    list_file_storage_tokens = list(set(str(x) for x in file_storage_tokens))
    deleted_file_storage_tokens = []
    try:
        """
        validation
        """
        # pass
        """
        main
        """
        for file_storage_token in list_file_storage_tokens:
            # get file path
            local_dict_file_row = get_file_row(file_storage_token)
            local_string_system_absolute_file_path = str(
                os.path.join(
                    global_absolute_path_local_storage,
                    os.sep.join(
                        local_dict_file_row[File.file_system_relative_path.name].split(
                            "/"
                        )
                    ),
                    local_dict_file_row[File.file_system_file_name_with_extension.name],
                )
            )

            # delete file
            if os.path.exists(local_string_system_absolute_file_path):
                os.remove(local_string_system_absolute_file_path)
            else:
                global_object_square_logger.logger.warning(
                    f"file {local_string_system_absolute_file_path} should have existed but doesn't."
                )

            deleted_file_storage_tokens.append(file_storage_token)
        local_object_square_database_helper.delete_rows_v0(
            database_name=global_string_database_name,
            schema_name=global_string_schema_name,
            table_name=File.__tablename__,
            filters=FiltersV0(
                root={
                    File.file_storage_token.name: FilterConditionsV0(
                        in_=deleted_file_storage_tokens
                    ),
                }
            ),
        )
        """
        return value
        """
        output_content = get_api_output_in_standard_format(
            data={"main": deleted_file_storage_tokens},
            message=messages["GENERIC_DELETE_SUCCESSFUL"],
        )
        return JSONResponse(content=output_content)
    except HTTPException as http_exception:
        """
        rollback logic
        """
        # pass
        global_object_square_logger.logger.error(http_exception, exc_info=True)
        return JSONResponse(
            status_code=http_exception.status_code,
            content=http_exception.detail,
        )
    except Exception as e:
        """
        rollback logic
        """
        # pass
        global_object_square_logger.logger.error(e, exc_info=True)
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_500"],
            log=str(e),
        )
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            content=output_content,
        )



from fastapi import status
from fastapi.exceptions import HTTPException
from square_commons import get_api_output_in_standard_format
from square_database_helper import SquareDatabaseHelper, FiltersV0
from square_database_helper.pydantic_models import FilterConditionsV0
from square_database_structure.square import global_string_database_name
from square_database_structure.square.file_storage import global_string_schema_name
from square_database_structure.square.file_storage.tables import File

from square_file_store.configuration import (
    config_str_square_database_protocol,
    config_str_square_database_ip,
    config_int_square_database_port,
    global_object_square_logger,
)
from square_file_store.messages import messages

local_object_square_database_helper = SquareDatabaseHelper(
    param_str_square_database_ip=config_str_square_database_ip,
    param_str_square_database_protocol=config_str_square_database_protocol,
    param_int_square_database_port=config_int_square_database_port,
)


@global_object_square_logger.auto_logger()
def create_entry_in_file_store(
    file_name_with_extension: str,
    content_type: str,
    system_file_name_with_extension: str,
    file_storage_token: str,
    app_id: int,
    system_relative_path: str,
):
    try:

        data = [
            {
                File.file_name_with_extension.name: file_name_with_extension,
                File.file_content_type.name: content_type,
                File.file_system_file_name_with_extension.name: system_file_name_with_extension,
                File.file_system_relative_path.name: system_relative_path,
                File.file_storage_token.name: file_storage_token,
                File.app_id.name: app_id,
            }
        ]

        response = local_object_square_database_helper.insert_rows_v0(
            data=data,
            database_name=global_string_database_name,
            schema_name=global_string_schema_name,
            table_name=File.__tablename__,
        )["data"]["main"]

        return response
    except Exception as e:
        raise e


@global_object_square_logger.auto_logger()
def get_file_row(file_storage_token):
    try:
        response = local_object_square_database_helper.get_rows_v0(
            database_name=global_string_database_name,
            schema_name=global_string_schema_name,
            table_name=File.__tablename__,
            filters=FiltersV0(
                root={
                    File.file_storage_token.name: FilterConditionsV0(
                        eq=file_storage_token
                    ),
                }
            ),
        )["data"]["main"]
        if len(response) != 1:
            output_content = get_api_output_in_standard_format(
                message=messages["GENERIC_400"],
                log=f"incorrect file_storage_token: {file_storage_token}.",
            )
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=output_content,
            )
        return response[0]
    except Exception as e:
        raise e



import os
import sys

from square_commons import ConfigReader
from square_logger.main import SquareLogger

try:

    config_file_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "data", "config.ini"
    )
    config_sample_file_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "data", "config.sample.ini"
    )
    ldict_configuration = ConfigReader(
        config_file_path, config_sample_file_path
    ).read_configuration()

    # get all vars and typecast
    # ===========================================
    # general
    config_str_module_name = ldict_configuration["GENERAL"]["MODULE_NAME"]
    # ===========================================

    # ===========================================
    # environment
    config_str_host_ip = ldict_configuration["ENVIRONMENT"]["HOST_IP"]
    config_int_host_port = int(ldict_configuration["ENVIRONMENT"]["HOST_PORT"])
    config_list_allow_origins = eval(
        ldict_configuration["ENVIRONMENT"]["ALLOW_ORIGINS"]
    )

    config_str_log_file_name = ldict_configuration["ENVIRONMENT"]["LOG_FILE_NAME"]
    config_str_local_storage_folder_path = ldict_configuration["ENVIRONMENT"][
        "LOCAL_STORAGE_PATH"
    ]
    config_str_ssl_crt_file_path = ldict_configuration["ENVIRONMENT"][
        "SSL_CRT_FILE_PATH"
    ]
    config_str_ssl_key_file_path = ldict_configuration["ENVIRONMENT"][
        "SSL_KEY_FILE_PATH"
    ]

    config_str_db_ip = ldict_configuration["ENVIRONMENT"]["DB_IP"]
    config_int_db_port = int(ldict_configuration["ENVIRONMENT"]["DB_PORT"])
    config_str_db_username = ldict_configuration["ENVIRONMENT"]["DB_USERNAME"]
    config_str_db_password = ldict_configuration["ENVIRONMENT"]["DB_PASSWORD"]

    # ===========================================

    # ===========================================
    # square_logger
    config_int_log_level = int(ldict_configuration["SQUARE_LOGGER"]["LOG_LEVEL"])
    config_str_log_path = ldict_configuration["SQUARE_LOGGER"]["LOG_PATH"]
    config_int_log_backup_count = int(
        ldict_configuration["SQUARE_LOGGER"]["LOG_BACKUP_COUNT"]
    )
    config_formatter_choice = ldict_configuration["SQUARE_LOGGER"]["FORMATTER_CHOICE"]
    if config_formatter_choice not in ("human_readable", "json"):
        raise ValueError(f"Invalid formatter choice: {config_formatter_choice}")
    config_bool_enable_redaction = eval(
        ldict_configuration["SQUARE_LOGGER"]["ENABLE_REDACTION"]
    )
    # ===========================================
    # ===========================================
    # square_database_helper

    config_str_square_database_protocol = ldict_configuration["SQUARE_DATABASE_HELPER"][
        "SQUARE_DATABASE_PROTOCOL"
    ]
    config_str_square_database_ip = ldict_configuration["SQUARE_DATABASE_HELPER"][
        "SQUARE_DATABASE_IP"
    ]
    config_int_square_database_port = int(
        ldict_configuration["SQUARE_DATABASE_HELPER"]["SQUARE_DATABASE_PORT"]
    )
    # ===========================================
    # initialize logger

    global_object_square_logger = SquareLogger(
        log_file_name=config_str_log_file_name,
        log_level=config_int_log_level,
        log_path=config_str_log_path,
        log_backup_count=config_int_log_backup_count,
        formatter_choice=config_formatter_choice,
        enable_redaction=config_bool_enable_redaction,
    )
except Exception as e:
    print(
        "\033[91mMissing or incorrect config.ini file.\n"
        "Error details: " + str(e) + "\033[0m"
    )
    sys.exit()

# extra logic for this module

try:
    global_absolute_path_local_storage = os.path.abspath(
        config_str_local_storage_folder_path
    )
    if not os.path.exists(global_absolute_path_local_storage):
        os.makedirs(global_absolute_path_local_storage)
except Exception as e:
    print(
        "\033[91mIncorrect value for LOCAL_STORAGE_PATH in config.ini file.\n"
        "Error details: " + str(e) + "\033[0m"
    )
    sys.exit()



import io


def upload_document(create_client_and_cleanup):
    client = create_client_and_cleanup
    file_content = b"Hello, this is the content of the file."
    file_name = "example.txt"

    # Create an in-memory byte stream
    file = io.BytesIO(file_content)

    files = {"file": (file_name, file, "multipart/form-data")}

    # Make the request
    response = client.post("/upload_file/v0", files=files)

    return response.json()


def test_read_main(create_client_and_cleanup):
    #################################
    # download file test case
    #################################
    client = create_client_and_cleanup

    upload_result = upload_document(client)
    uploaded_file_storage_token = upload_result["data"]["main"]
    parameter = {"file_storage_token": uploaded_file_storage_token}

    # Make the request
    response = client.get("/download_file/v0", params=parameter)

    # Assert the response status code
    assert response.status_code == 200



from square_commons import get_api_output_in_standard_format


def test_read_main(get_patched_configuration, create_client_and_cleanup):
    client = create_client_and_cleanup
    response = client.get("/")
    assert response.status_code == 200
    assert response.json() == get_api_output_in_standard_format(
        log=get_patched_configuration.config_str_module_name
    )



import io


def test_read_main(create_client_and_cleanup):
    #################################
    # upload file test case
    #################################
    client = create_client_and_cleanup

    file_content = b"Hello, this is the content of the file."
    file_name = "example.txt"

    # Create an in-memory byte stream
    file = io.BytesIO(file_content)

    files = {"file": (file_name, file, "multipart/form-data")}

    # Make the request
    response = client.post("/upload_file/v0", files=files)

    # Assert the response status code
    assert response.status_code == 201
    upload_response = response.json()
    assert "data" in upload_response and "main" in upload_response["data"]



from typing import Annotated, List, Optional
from uuid import UUID

from fastapi import APIRouter, Form, UploadFile, status
from fastapi.exceptions import HTTPException
from fastapi.params import Query
from fastapi.responses import JSONResponse
from square_commons import get_api_output_in_standard_format

from square_file_store.configuration import (
    global_object_square_logger,
)
from square_file_store.messages import messages
from square_file_store.utils.routes.core import (
    util_upload_file_v0,
    util_download_file_v0,
    util_delete_files_v0,
)

router = APIRouter(
    tags=["core"],
)


@router.post("/upload_file/v0", status_code=status.HTTP_201_CREATED)
@global_object_square_logger.auto_logger()
async def upload_file_v0(
    file: UploadFile,
    app_id: Annotated[
        Optional[int],
        Form(title="App id", description="Specify the app id"),
    ] = None,
    system_relative_path: Annotated[
        str,
        Form(
            title="System relative path",
            description="Specify the path using '/'. For e.g. home/user_document",
        ),
    ] = "others/misc",
):
    try:
        return await util_upload_file_v0(
            file=file, app_id=app_id, system_relative_path=system_relative_path
        )
    except HTTPException as he:
        global_object_square_logger.logger.error(he, exc_info=True)
        return JSONResponse(status_code=he.status_code, content=he.detail)
    except Exception as e:
        global_object_square_logger.logger.error(e, exc_info=True)
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_500"], log=str(e)
        )
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, content=output_content
        )


@router.get("/download_file/v0", status_code=status.HTTP_200_OK)
@global_object_square_logger.auto_logger()
async def download_file_v0(file_storage_token: UUID):
    try:
        return util_download_file_v0(
            file_storage_token=file_storage_token,
        )
    except HTTPException as he:
        global_object_square_logger.logger.error(he, exc_info=True)
        return JSONResponse(status_code=he.status_code, content=he.detail)
    except Exception as e:
        global_object_square_logger.logger.error(e, exc_info=True)
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_500"], log=str(e)
        )
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, content=output_content
        )


@router.delete("/delete_files/v0", status_code=status.HTTP_200_OK)
@global_object_square_logger.auto_logger()
async def delete_files_v0(file_storage_tokens: List[UUID] = Query()):
    try:
        return util_delete_files_v0(
            file_storage_tokens=file_storage_tokens,
        )
    except HTTPException as he:
        global_object_square_logger.logger.error(he, exc_info=True)
        return JSONResponse(status_code=he.status_code, content=he.detail)
    except Exception as e:
        global_object_square_logger.logger.error(e, exc_info=True)
        output_content = get_api_output_in_standard_format(
            message=messages["GENERIC_500"], log=str(e)
        )
        return JSONResponse(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, content=output_content
        )



messages = {
    "GENERIC_CREATION_SUCCESSFUL": "records created successfully.",
    "GENERIC_READ_SUCCESSFUL": "data retrieved successfully.",
    "GENERIC_UPDATE_SUCCESSFUL": "your information has been updated successfully.",
    "GENERIC_DELETE_SUCCESSFUL": "your records have been deleted successfully.",
    "GENERIC_400": "the request is invalid or cannot be processed.",
    "GENERIC_500": "an internal server error occurred. please try again later.",
}



from setuptools import find_packages, setup

package_name = "square_file_store"

setup(
    name=package_name,
    version="2.3.5",
    packages=find_packages(),
    package_data={
        package_name: ["data/*"],
    },
    install_requires=[
        "uvicorn>=0.24.0.post1",
        "fastapi>=0.104.1",
        "python-multipart>=0.0.6",
        "pydantic>=2.5.3",
        "pytest>=8.0.0",
        "square_logger>=3.0.0",
        "square_database_helper>=2.3.0",
        "square_commons>=2.1.0",
        "square_database_structure>=2.5.1",
        "httpx>=0.27.2",
    ],
    author="Parth Mukesh Mangtani",
    author_email="thepmsquare@gmail.com",
    description="file storage layer for my personal server.",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url=f"https://github.com/thepmsquare/{package_name}",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Utilities",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Framework :: FastAPI",
    ],
)
